﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Simplify_Xr
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void btnSHP_Click(object sender, EventArgs e)
        {
            DialogResult DR = openFileDialog.ShowDialog();
            if (DR == DialogResult.OK)
            {
                tbSHP.Text = openFileDialog.FileName;
            }
        }

        private void btnOut_Click(object sender, EventArgs e)
        {
            DialogResult DR = saveFileDialog.ShowDialog();
            if (DR == DialogResult.OK)
            {
                tbOut.Text = saveFileDialog.FileName;
            }
        }

        private XrMapLib.CoordinateListCollection GetShapeCoordList(XrMapLib.IShapeRow row)
        {
            XrMapLib.CoordinateListCollection CLC = new XrMapLib.CoordinateListCollection();

            int cntParts = row.PartCount;
            for (int iPart = 0; iPart < cntParts; ++iPart)
            {
                int cntVertices = row.GetVertexCountFromPart(iPart);
                XrMapLib.CoordinateList CL = new XrMapLib.CoordinateList();
                if (cntVertices == 0) return null;
                for (int iVertex = 0; iVertex < cntVertices; ++iVertex)
                {
                    XrMapLib.Coordinate coord = row.GetVertexFromPart(iPart, iVertex);
                    CL.Add(coord.X, coord.Y);
                }

                CLC.Add(CL);
            }

            return CLC;
        }

        private XrMapLib.FieldSetDefine GetFieldDefine(XrMapLib.IAttributeTable table)
        {
            XrMapLib.FieldSet fieldSet = table.FieldSet;

            XrMapLib.FieldSetDefine FSD = new XrMapLib.FieldSetDefine();
            int cntFields = fieldSet.FieldCount;
            for (int iField = 0; iField < cntFields; ++iField)
            {
                XrMapLib.XrFieldTypeEnum type = fieldSet.GetFieldType(iField);
                String fieldName = fieldSet.GetFieldName(iField);
                if (type == XrMapLib.XrFieldTypeEnum.XrDouble || type == XrMapLib.XrFieldTypeEnum.XrFloat)
                {
                    FSD.AddFloatField(fieldName, 10, 2);
                }
                else if (type == XrMapLib.XrFieldTypeEnum.XrInteger || type == XrMapLib.XrFieldTypeEnum.XrShort || type == XrMapLib.XrFieldTypeEnum.XrVeryShort)
                {
                    FSD.AddIntegerField(fieldName, 10);
                }
                else if (type == XrMapLib.XrFieldTypeEnum.XrString)
                {
                    FSD.AddStringField(fieldName, 25);
                }
            }

            return FSD;
        }

        private XrMapLib.ValueList GetAttributeValueList(XrMapLib.IAttributeRow row, int fieldsCount)
        {
            XrMapLib.ValueList values = new XrMapLib.ValueList();

            for (int iField = 0; iField < fieldsCount; ++iField)
            {
                String value = row.GetValueAsString(iField);
                values.AddValueByString(value);
            }

            return values;
        }

        private void btnRun_Click(object sender, EventArgs e)
        {
            Xr.Layers.RemoveAllLayer();
            Xr.Layers.AddShapeMapLayer("lyr", tbSHP.Text);
            Xr.WaitForAllConnections();

            double Tol = Double.Parse(tbTol.Text);

            XrMapLib.ShapeMapLayer SML = Xr.Layers.GetLayerAsShapeMap("lyr");
            if (SML != null)
            {
                XrMapLib.ShapeTable ST = SML.ShapeTable;
                XrMapLib.AttributeTable AT = SML.AttributeTable;
                XrMapLib.Tool tool = new XrMapLib.Tool();
                XrMapLib.FieldSetDefine FSD = GetFieldDefine(AT);

                if (ST.ShapeType != XrMapLib.XrShapeTypeEnum.XrPolygon && ST.ShapeType != XrMapLib.XrShapeTypeEnum.XrPolyline)
                {
                    MessageBox.Show("ERR: SHP TYPE IS POINT");
                    return;
                }

                XrMapLib.EsriShpFileHandler SFH = tool.CreateEsriShpFileHandler(tbOut.Text, ST.ShapeType, FSD);
                if (SFH == null)
                {
                    MessageBox.Show("ERR : CREATING SHP FILE");
                    return;
                }

                int RowCount = ST.RowCount;
                progressBar.Maximum = RowCount;
                progressBar.Value = 0;
                int FieldCount = AT.FieldSet.FieldCount;
                XrMapLib.ValueList VL = new XrMapLib.ValueList();
                for (int FID = 0; FID < RowCount; ++FID)
                {
                    XrMapLib.ShapeRow SR = ST.GetRow(FID);
                                     XrMapLib.IAttributeRow AR = AT.GetRow(FID);
                    if (SR.Load() && AR.Load())
                    {
                        VL = GetAttributeValueList(AR, FieldCount);
                        XrMapLib.SpatialOperator ISO = SR.ToSpatialOperator();
                        XrMapLib.SpatialOperator OSO = ISO.Simplify(Tol);

                        if (SR.FromSpatialOperator(OSO))
                        {
                            XrMapLib.CoordinateListCollection CLC = GetShapeCoordList(SR);
                            if (CLC == null)
                            {
                                if(ISO.SimpleGeometry) lbResult.Items.Add("ID: " + FID);
                                else lbResult.Items.Add("ID: " + FID + "(Not Simple");
                            }
                            else if (!SFH.AddMultiPart(CLC, VL))
                            {
                                MessageBox.Show("ERR: ADD POINTS");
                                SFH.Close();
                                return;
                            }
                        }
                        else
                        {
                            MessageBox.Show("ERR: FAILED SIMPLIFY");
                            SFH.Close();
                            return;
                        }

                        AR.Unload();
                        SR.Unload();

                        progressBar.PerformStep();
                        Application.DoEvents();
                    }
                }

                SFH.Close();
            }

            MessageBox.Show("Done");
        }
    }
}
